package Seiros::OpenVZ::database;

#=============================================================================#
#			Database cache with OpenVZ info			      #
#=============================================================================#
# (C) Denis Smirnov <mithraen@freesource.info)                    04 Nov 2007 #
#=============================================================================#
use 5.008;
use strict;
use warnings;

use DBI;
use DBD::SQLite;

require Exporter;

our @ISA = qw(Exporter);

our $VERSION = '0.01';

my %dbq;

$dbq{add} =
"INSERT INTO user_beancounters (uid, resource, held, maxheld, barrier,  'limit', failcnt) VALUES (?*1, ?, ?*1, ?*1, ?*1, ?*1, ?*1)";
$dbq{select_veids} = "SELECT uid FROM user_beancounters GROUP BY uid";
$dbq{select_bugs}  =
  'SELECT uid,resource,maxheld,"limit",(max("limit"*1.5,maxheld*1.5))
			FROM user_beancounters
			WHERE  maxheld/"limit" > 0.8';
$dbq{select_bugs_min} =
'select uid,resource,maxheld,"limit",round(maxheld*2+1) from user_beancounters,vz
      WHERE
              uid > 0
      AND
              user_beancounters.uid = vz_id
      AND
              maxheld > 0
      AND
              maxheld < "limit" * 0.2';

$dbq{is_veid}        = 'SELECT count(*) FROM vz WHERE vz_id = ?';
$dbq{get_ve_name}    = 'SELECT vz_name FROM vz WHERE vz_id = ?';
$dbq{update_ve_name} = 'UPDATE vz SET vz_name = ? WHERE vz_id = ?';
$dbq{add_ve_name}    = 'INSERT INTO vz (vz_id, vz_name) VALUES (?, ?)';

sub new {
    my $self = {
        'dbh' => undef,
        'sth' => {},
        'stq' => {}
    };
    bless $self;
    return $self;
}

# Update VE list from real configuration
sub update_ve_list($$) {
    my $this = shift;
    my $cfg  = shift;

    # Get list for update
    my %veid2name = %{ $cfg->{veid2name} };

    # Add missed records
    foreach ( sort keys %veid2name ) {
        $_ =~ /^(\d+)$/ or die;
        $_ = $1;
        $veid2name{$_} =~ /^([a-zA-Z0-9\.\_]+)$/;
        $veid2name{$_} = $1;
        $this->{sth}->{add_ve_name}->execute( $_, $veid2name{$_} );
    }

}

sub check_ints(@) {
    my @rc;
    push @rc, check_int($_) foreach @_;
    return @rc;
}

sub check_int($) {
    return undef unless $_[0] =~ /^(\d+)$/;
    return $1;
}

sub user_beancounter_add {
    my $this     = shift;
    my $veid     = check_int(shift);
    my $resource = shift;
    $resource =~ /^([a-z]+)$/;
    $resource = $1;
    my ( $held, $maxheld, $barrier, $limit, $failcnt ) = check_ints(@_);
    $this->{sth}->{add}
      ->execute( $veid, $resource, $held, $maxheld, $barrier, $limit,
        $failcnt );
    $this->{sth}->{add}->finish();
}

sub list_bugs($) {
    my $this = shift;
    my $sth  = $this->{sth}->{select_bugs};
    $sth->execute();
    return @{ $sth->fetchall_arrayref };
}

sub list_bugs_min($) {
    my $this = shift;
    my $sth  = $this->{sth}->{select_bugs_min};
    $sth->execute();
    return @{ $sth->fetchall_arrayref };
}

sub list_veids($) {
    my $this = shift;
    my $sth  = $this->{sth}->{select_veids};
    $sth->execute();
    my $rc = $sth->fetchall_arrayref;
    my @rc;
    push @rc, $_->[0] foreach @{$rc};
    return @rc;
}

sub init_db($$) {
    my $this   = shift;
    my $dbname = shift || die "Database name not defined\n";

    #    
    unlink $dbname;
    $this->{dbh} =
      DBI->connect( "dbi:SQLite:dbname=$dbname", "", "",
        { RaiseError => 0, AutoCommit => 1 } )
      || die;

    # We need to work FAST, that means VERY FAST
    $this->{dbh}->do("PRAGMA synchronous = OFF") || die;
    $this->{dbh}->do("PRAGMA locking_mode = EXCLUSIVE")
      || die "Can't lock database '$dbname'\n";

    # Create table if not exists
    $this->{dbh}->do(
        "CREATE TABLE user_beancounters(
			    uid,
				resource,
				held,
				maxheld,
				barrier,
				'limit',
				failcnt
				)"
    );
    $this->{dbh}->do(
        "CREATE TABLE vz (
	    vz_id       INTEGER PRIMARY KEY,
            vz_name     VARCHAR,
	    vz_hostname VARCHAR,
	    vz_ip       VARCHAR
	)"
    );

    #	$this->{dbh}->begin_work();
    # clean user_beancounters content
    $this->{dbh}->do("DELETE FROM user_beancounters");

    foreach ( keys %dbq ) {
        $this->{sth}->{$_} = $this->{dbh}->prepare( $dbq{$_} )
          || die "Can't prepare '$_'\n";
    }
}

1;
